/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2012, 2021 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_main.c
* Version      : CodeGenerator for RL78/F14 V2.03.07.02 [08 Nov 2021]
* Device(s)    : R5F10PGG
* Tool-Chain   : CCRL
* Description  : This file implements main function.
* Creation Date: 2022/11/16
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_serial.h"
#include "r_cg_timer.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_typedefs.h"
#include "r_cg_userdefine.h"
#include "fsl.h"
#include "fsl_types.h"
#include "r_xmodem.h"
#include "crc16.h"
#include "user_def.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#define PRODUCTION_PROGRAM_REV_MAJOR   0
#define PRODUCTION_PROGRAM_REV_MINOR   1

typedef enum
{
	FSL_INIT,
	FSL_ERASE,
	FSL_WRITE,
	FSL_INVERTBOOTFLAG
} error_handler_case;


void ErrorHandler(uint8_t case_num);
void delay(uint32_t delay_time);
void R_MAIN_UserInit(void);
void R_UART0_Send_String(__far int8_t buffer[]) ;
void R_UART0_Send_Byte(int8_t data) ;

extern fsl_u08 WriteFlash(uint32_t addr, int8_t data[], uint8_t size);
int size_of ( int8_t buffer1[]);
uint8_t GetByte (uint32_t timeout, int8_t *data);
/***********************************************************************************************************************
* Function Name: main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void main(void)
{

	
	uint32_t 	i = 0;
  	int8_t 		selection;
  	int8_t 		reselect = 1;
  	fsl_u08 	fsl_status;
  	__far fsl_descriptor_t 	fsl_descr;
	uint8_t		ret;
	uint32_t	flag_start = 0xFFFF;
	uint32_t	flag_end = 0xFFFF;
	uint32_t	flag_start_end = 0xFFFF;
	unsigned short		CRC_rom, CRC_ccitt;	//WangJin 2021.02.01
	uint8_t			OTA_FLAG_START[4] = {0xAA,0xFF,0xFF,0xFF};
	uint8_t			OTA_FLAG_END[4] = {0x55,0xFF,0xFF,0xFF};
	
    /* Start user code. Do not edit comment generated here */
	//EI();		//WangJin 2021.01.20
//start: only for test by hh 2022.08.08
	CRC_ccitt = *(__far unsigned int *)CRC_ADDRESS;
	CRC_rom = calcrc(FLASH_AREA, (CRC_ADDRESS - FLASH_AREA));		//0x2000 - 0xFBF3
//end: only for test
    /****************/
    /* Boot Program */
    /****************/
    R_UART0_Start();
            
	/* initialize self-programming environment */
	fsl_descr.fsl_flash_voltage_u08 = 0x00;
	fsl_descr.fsl_frequency_u08 = 0x08;
	fsl_descr.fsl_auto_status_check_u08 = 0x01;
	
	fsl_status = FSL_Init(&fsl_descr);
	
	if(fsl_status != FSL_OK)
	{
		ErrorHandler(FSL_INVERTBOOTFLAG);
	}
	
	FSL_Open();
	FSL_PrepareFunctions();
	FSL_PrepareExtFunctions();

	/* Print operation menu */

#if 1
	R_UART0_Send_String((__far int8_t*)"\n\r*** FW Ver.: 1.5  old ***\n\r");
	//R_UART0_Send((uint8_t *)message3, 50U);
#else
	R_UART0_Send_String((__far int8_t*)"\n\r*** FW Ver.: 1.6  new ***\n\r");
#endif


	while (1){
		
		R_UART0_Send_String((__far int8_t*)"\n\rChoose actions:(1)Update Boot area + App area (2)Update App area only\n\r");
/*
		while (1){
			WDTE = 0xAC;
			if (GetByte(1000, &selection)) break;
		}
		*/
		if (FALSE == GetByte(5000,  &selection)) break;
#if 0 // only for test FSL  ****start
		NOP();
		NOP();
		
		for (i = 4; i < FLASH_END_BLOCK; i++) 
		{
			fsl_status = FSL_Erase(i);
			if (fsl_status != FSL_OK)
			{
				ErrorHandler(FSL_ERASE);
			}
		}
				
		if( FSL_OK != WriteFlash(OTA_FLAG_START_ADDRESS, (int8_t*)OTA_FLAG_START, 4))
		{
			ErrorHandler(FSL_ERASE);
		}
		while(1);
#endif // only for test FSL ****end
		switch (selection){
			case '1':
				R_UART0_Send_String((__far int8_t*)"\n\rSelect file to write into flash area.");
                                    
				for (i = 8; i < FLASH_END_BLOCK; i++) 
				{
					fsl_status = FSL_Erase(i);
					if (fsl_status != FSL_OK)
					{
						ErrorHandler(FSL_ERASE);
					}
				}
				
				if( FSL_OK != WriteFlash(OTA_FLAG_START_ADDRESS, (int8_t*)OTA_FLAG_START, 4))
				{
					ErrorHandler(FSL_ERASE);
				}
				
				fsl_status = XmodemProg(BOOT_AREA);

				if(fsl_status == XM_OK)
				{
					CRC_ccitt = *(__far unsigned int *)CRC_ADDRESS;
					CRC_rom = calcrc(FLASH_AREA, (CRC_ADDRESS - FLASH_AREA));
					
					if( CRC_ccitt == CRC_rom )
					{
						if( FSL_OK == WriteFlash(OTA_FLAG_END_ADDRESS, (int8_t*)OTA_FLAG_END, 4) )
						{
							R_UART0_Send_String((__far int8_t*)"\n\rUpdate boot area: Succeeded");
							//R_UART0_Send_String((__far int8_t*)"\n\rGoto app.\n\r");
						}
						else
						{
							ErrorHandler(FSL_ERASE);
						}
					}
					else
					{
						R_UART0_Send_String((__far int8_t*)"\n\rCRC NG\n\r");
						FSL_ForceReset();
					}
					fsl_status = FSL_InvertBootFlag();
					if (fsl_status != FSL_OK)
					{
						ErrorHandler(FSL_INVERTBOOTFLAG);
					}
					else
					{
						R_UART0_Send_String((__far int8_t*)"\n\rInvertBootFlag: Succeeded");
					}
				}
				else
				{
					R_UART0_Send_String((__far int8_t*)"\n\rUpdate boot + app area: Failed");
					FSL_ForceReset();
				}
				R_UART0_Send_String((__far int8_t*)"\n\rReset\n\r");
				reselect = 0;
				//FSL_ForceReset();
				break;
			
			case '2':
				R_UART0_Send_String((__far int8_t*)"\n\rPlease select flash.bin file\n\r");

				for (i = FLASH_AREA/FLASH_BYTE_PER_BLOCK; i < FLASH_END_BLOCK; i++)
				{
					fsl_status = FSL_Erase(i);
					if (fsl_status != FSL_OK)
					{
						ErrorHandler(FSL_ERASE);
					}
				}
				
				if( FSL_OK != WriteFlash(OTA_FLAG_START_ADDRESS, (int8_t*)OTA_FLAG_START, 4))
//				if( FSL_OK != WriteFlash(OTA_FLAG_END_ADDRESS, (int8_t*)OTA_FLAG_START, 4))
//				if( FSL_OK != WriteFlash(CRC_ADDRESS, (int8_t*)OTA_FLAG_START, 4))
				{
					ErrorHandler(FSL_ERASE);
				}
				
				fsl_status = XmodemProg_FlashOnly(FLASH_AREA);

				if(fsl_status == XM_OK)
				{
					CRC_ccitt = *(__far unsigned int *)CRC_ADDRESS;//1fbfc
					CRC_rom = calcrc(FLASH_AREA, (CRC_ADDRESS - FLASH_AREA));
					
					if( CRC_ccitt == CRC_rom )
					{
						if( FSL_OK == WriteFlash(OTA_FLAG_END_ADDRESS, (int8_t*)OTA_FLAG_END, 4) )
						{
							R_UART0_Send_String((__far int8_t*)"\n\rUpdate flash area: Succeeded");
							R_UART0_Send_String((__far int8_t*)"\n\rGoto app.\n\r");
							reselect = 0;
						}
						else
						{
							R_UART0_Send_String((__far int8_t*)"\n\rCRC NG\n\r");
							ErrorHandler(FSL_ERASE);
						}
					}
				}
				else
				{
					R_UART0_Send_String((__far int8_t*)"\n\rUpdate flash area: Failed");
					FSL_ForceReset();
//					while (1)
//					{
//						WDTE = 0xAC;
//					}
				}
				break;
			default:
				R_UART0_Send_String((__far int8_t*)"\n\rSelect again.\n\r");
    			reselect = 1;
				break;
			}

		if(reselect == 0)
			return;
    	}
	
		flag_start = *(__far unsigned int *)OTA_FLAG_START_ADDRESS;	//for sample OTA_FLAG_START_ADDRESS= 0xFBF8 [AA]
		flag_end = *(__far unsigned int *)OTA_FLAG_END_ADDRESS;			//for sample OTA_FLAG_END_ADDRESS= 0xFBFC  [55]
		flag_start_end = ((flag_start & 0x00FF) << 8) | (flag_end & 0x00FF);
	
		if( 0xAAFF == flag_start_end ){		//The OTA operation has been started
			for (i = 4; i < FLASH_END_BLOCK; i++)
			{
				fsl_status = FSL_Erase(i);
				if (fsl_status != FSL_OK)
				{
					ErrorHandler(FSL_ERASE);
					//goto loop;
				}
			}
		}
		else if( 0xFFFF == flag_start_end ){	//There has benn no OTA operation launched
			CRC_ccitt = *(__far unsigned int *)CRC_ADDRESS;
			CRC_rom = calcrc(FLASH_AREA, (CRC_ADDRESS - FLASH_AREA));

			if(CRC_rom == CRC_ccitt){	//The image in the flash is available
				R_UART0_Send_String((__far int8_t*)"\n\rjump to 0x2000\n\r");
//				goto jump_to_app;
				return;
			}
			else{						//The image in the flash is unavailable
				R_UART0_Send_String((__far int8_t*)"\n\rimage is damaged\n\r");
				//goto loop;
			}
		}
		else if( 0xAA55 == flag_start_end ){	//The previous OTA operation succeeded
			//goto jump_to_app;
			return;
		}
		else{
			FSL_ForceReset();
		}
    	
    	fsl_status = FSL_BlankCheck(FLASH_AREA/FLASH_BYTE_PER_BLOCK);
    	if (fsl_status == FSL_OK)
    	{
    		R_UART0_Send_String((__far int8_t*)"\n\rThe user program is empty.");
    		delay(3000);
    		//FSL_ForceReset();
    	}
/*
jump_to_app:
	;	*/
	NOP();

    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: R_MAIN_UserInit
* Description  : This function adds user code before implementing main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_MAIN_UserInit(void)
{
    /* Start user code. Do not edit comment generated here */
    EI();
    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
void R_UART0_Send_Byte(int8_t data) 
{ 
	if ((SRIF0) && (SSR00 != 0))        /* if any error in status register */
	{ 
		SRIF0 = 0;      				/* clear error interrupt request flag */ 
		SIR00 |= (SSR00&0x000F);
	} 

	SDR00L = data;     					/* load character to transmit register */
	while (STIF0 == 0)					/* wait while transmission in progress */
	{
	}
	STIF0 = 0;       					/* clear transmission interrupt request flag */
}

void R_UART0_Send_String(__far int8_t buffer[]) 
{ 
	int8_t i;
	
	for (i = 0; buffer[i] != 0; i++)
	{
		R_UART0_Send_Byte(buffer[i]); 
	}
}
int size_of ( int8_t buffer1[])
{
		int8_t i;
	
	for (i = 0; buffer1[i] != '\n'; i++)
	{
	  ; 
	 
	}
	return i;
}

void ErrorHandler(uint8_t case_num)
{
#if 0
	switch (case_num)
	{
		case FSL_INIT:
			R_UART0_Send_String((__far int8_t*)"\n\r FSL_Init failed");
			break;
		case FSL_ERASE:
			R_UART0_Send_String((__far int8_t*)"\n\r FSL_Erase failed");
			break;
		case FSL_WRITE:
			R_UART0_Send_String((__far int8_t*)"\n\r FSL_Write failed");
			break;
		case FSL_INVERTBOOTFLAG:
			R_UART0_Send_String((__far int8_t*)"\n\r FSL_InvertBootFlag failed");
			break;
	}
#endif
	//R_UART0_Send(( int8_t*)message3,size_of(message3));
	FSL_ForceReset();
//	while (1)
//	{
//		WDTE = 0xAC;
//	}
}
void delay(uint32_t delay_time)
{
	R_TAU0_Channel0_Start();
	while (delay_time)
	{
		if (TMIF00 == 1)
		{
			WDTE = 0xAC;
			TMIF00 = 0;
			delay_time--;
		}
	}
	R_TAU0_Channel0_Stop();
}